﻿using System;

using Iterator.Units;
using Iterator.Buildings;
using Iterator.Buildings.GDI;
using Iterator.Buildings.NOD;
using Iterator.Buildings.Upgrades;
using Iterator.Hardware;
using Iterator.Screens;
using Iterator.Commands;
using Iterator.Adapters;
using Iterator.Storage;
using Iterator.Storage.ComplexSubsystem;
using Iterator.Graphics;

using ThirdParty;
using Iterator.CommandCenter;

namespace Iterator
{
    class Program
    {
        static void Main(string[] args)
        {
            #region Observer/Event Pattern Stuff
            /*
            GameUnit exampleUnit = new Soldier();

            BattleMapDisplay battleMap = new BattleMapDisplay(exampleUnit);
            OverviewDisplay overviewMap = new OverviewDisplay(exampleUnit);

            exampleUnit.Move(new Point(10, 10));
            exampleUnit.Move(new Point(20, 20));
            */
            #endregion

            #region Decorator Pattern Stuff
            Console.WriteLine("Decorator Pattern Stuff");
            Console.WriteLine("-----------------------");

            IBuilding gdiBarracks = new Barracks();
            Console.WriteLine(string.Format("{0} needs {1} power.",
                                            gdiBarracks.Description,
                                            gdiBarracks.Power));

            IBuilding buildingWithOneTurret = new BuildingTurret(new Barracks());
            Console.WriteLine(string.Format("{0} needs {1} power.", 
                                            buildingWithOneTurret.Description, 
                                            buildingWithOneTurret.Power));

            IBuilding aRealFortress = new BuildingTurret(
                                        new BuildingTurret(
                                            new BuildingSAM(
                                                new BuildingLaser(
                                                    new Barracks()
                                                    )
                                                    )
                                                    )
                                                    );

            Console.WriteLine(string.Format("{0} needs {1} power.", 
                                            aRealFortress.Description, 
                                            aRealFortress.Power));

            Console.WriteLine();
            #endregion

            #region Factory Method Stuff
            Console.WriteLine("Factory Method Pattern Stuff");
            Console.WriteLine("----------------------------");

            InfantryFactory barracks = new Barracks();
            InfantryFactory handOfNod = new HandOfNod();

            GameUnit eliteUnitGDI = barracks.BuildUnit(UnitType.EliteUnit);
            Console.WriteLine("Built a {0}, working for {1}, set to {2}.", 
                eliteUnitGDI.Description, 
                eliteUnitGDI.Side, 
                eliteUnitGDI.Behaviour);

            GameUnit eliteUnitNOD = handOfNod.BuildUnit(UnitType.EliteUnit);
            Console.WriteLine("Built a {0}, working for {1}, set to {2}.", 
                eliteUnitNOD.Description, 
                eliteUnitNOD.Side, 
                eliteUnitNOD.Behaviour);

            Console.WriteLine();
            #endregion

            #region Singleton Pattern Stuff
            Console.WriteLine("Singleton Pattern Stuff");
            Console.WriteLine("-----------------------");

            //GraphicsCard gfxCard = GraphicsCard.Instance;
            //GraphicsCardGeneric gfxCardGeneric = GraphicsCardGeneric.Instance;
            Console.WriteLine(GraphicsCard.Instance.Hello());
            Console.WriteLine(GraphicsCardGeneric.Instance.Hello());
            Console.WriteLine();
            #endregion

            #region Command Pattern Stuff
            Console.WriteLine("Command Pattern Stuff");
            Console.WriteLine("---------------------");

            // Our screen has some buttons in the right hand corner
            // Depending on the selected unit, it has different actions
            // Additionally, we want to have some recall last order functionality
            GameControls uiElement = new GameControls();

            // A soldier was selected, map the correct buttons
            GameUnit gdiSoldier = barracks.BuildUnit(UnitType.Soldier);
            ICommand stressSoldier = new StressUnit(gdiSoldier);
            ICommand relaxSoldier = new RelaxUnit(gdiSoldier);
            ICommand killSoldier = new SuicideUnit(gdiSoldier);
            uiElement.SetButton(0, stressSoldier);
            uiElement.SetButton(1, relaxSoldier);
            uiElement.SetButton(2, killSoldier);

            uiElement.ButtonPressed(0);
            uiElement.ButtonPressed(1);
            uiElement.UndoButtonPressed();

            uiElement.ButtonPressed(2);

            Console.WriteLine();
            // A building was selected, map the correct buttons
            ICommand powerDownBuilding = new PowerDownBuilding(aRealFortress);
            ICommand powerUpBuilding = new PowerUpBuilding(aRealFortress);
            uiElement.SetButton(0, powerDownBuilding);
            uiElement.SetButton(1, powerUpBuilding);

            uiElement.ButtonPressed(0);
            uiElement.ButtonPressed(1);
            uiElement.UndoButtonPressed();
            Console.WriteLine();
            #endregion

            #region Adapater Pattern Stuff
            Console.WriteLine("Adapater Pattern Stuff");
            Console.WriteLine("----------------------");

            Structure thirdPartyStructure = new GuardTower();
            IBuilding thirdPartyBuilding = new StructureAdapter(thirdPartyStructure);

            // From here on, our code is working with an IBuilding, 
            // the adapter is taking care of the conversion,
            // just like a real-world electricity adapter.

            Console.WriteLine(string.Format("{0} needs {1} power.",
                                            thirdPartyBuilding.Description,
                                            thirdPartyBuilding.Power));

            // A third party structure was selected, map the correct buttons
            ICommand powerDownStructure = new PowerDownBuilding(thirdPartyBuilding);
            ICommand powerUpStructure = new PowerUpBuilding(thirdPartyBuilding);
            uiElement.SetButton(0, powerDownStructure);
            uiElement.SetButton(1, powerUpStructure);

            Console.WriteLine();
            uiElement.ButtonPressed(0);
            Console.WriteLine(string.Format("{0} needs {1} power.",
                                            thirdPartyBuilding.Description,
                                            thirdPartyBuilding.Power));

            Console.WriteLine();
            uiElement.ButtonPressed(1);
            Console.WriteLine(string.Format("{0} needs {1} power.",
                                            thirdPartyBuilding.Description,
                                            thirdPartyBuilding.Power));

            Console.WriteLine();
            uiElement.UndoButtonPressed();
            Console.WriteLine(string.Format("{0} needs {1} power.",
                                            thirdPartyBuilding.Description,
                                            thirdPartyBuilding.Power));
            Console.WriteLine();
            #endregion

            #region Facade Pattern Stuff
            Console.WriteLine("Facade Pattern Stuff");
            Console.WriteLine("--------------------");

            // Should not be here, 
            // client should get a ready to use facade,
            // done here as a test now.
            HD hd = new HD();
            CD cd = new CD();
            StorageFacade storage = new StorageFacade(cd, hd);

            storage.LoadMap("Scorched Earth");
            Console.WriteLine();
            #endregion

            #region Template Method Pattern Stuff
            Console.WriteLine("Template Method Pattern Stuff");
            Console.WriteLine("-----------------------------");

            RenderEngine renderEngine2d = new RenderEngine2D();
            RenderEngine renderEngine3d = new RenderEngine3D();

            renderEngine2d.RenderScreen();
            Console.WriteLine();
            renderEngine3d.RenderScreen();
            Console.WriteLine();
            #endregion

            #region Iterator Pattern Stuff
            Console.WriteLine("Iterator Pattern Stuff");
            Console.WriteLine("----------------------");

            Eva gdiOrders = new Eva();
            Legion nodOrders = new Legion();

            gdiOrders.AddOrder("Defend", "Set up a base and defend it.");
            gdiOrders.AddOrder("Research", "Research new technology and deploy ion cannon.");

            nodOrders.AddOrder("Search and Destroy", "Roam the land and destroy everything.");
            nodOrders.AddOrder("Capture Uranium", "Capture a nuclear warhead.");
            nodOrders.AddOrder("Launch Nuke", "Launch nuke at GDI base.");

            WorldOrders worldOrders = new WorldOrders(gdiOrders, nodOrders);
            worldOrders.PrintWorldOrders();
            #endregion
        }
    }
}